import * as DynamoDB from 'aws-sdk/clients/dynamodb';
import { DocumentClient } from 'aws-sdk/clients/dynamodb';
import { ScanQueryOptions, ScanOptions, DynamoDBUtilOptions, QueryOptions } from './dynamodb-util.interface';
/**
 * ตัวช่วยทำ operation ต่างๆ บน DynamoDB
 */
export declare class DynamodbUtil {
    private documentClient;
    private readonly utilOptions;
    private readonly table;
    readonly raw: DocumentClient;
    constructor(table: string, utilOptions?: DynamoDBUtilOptions);
    /**
     * เรียกข้อมูลแบบ get item จาก DynamoDB
     * @param {object<[key: string]: string>} keys key หลักในการเรียกข้อมูลจากตาราง partitionKey หรือ compositeKey
     */
    get(keys: {
        [key: string]: string;
    }): Promise<DynamoDB.DocumentClient.AttributeMap | undefined>;
    /**
     * เก็บข้อมูลแบบ put item จาก DynamoDB
     * @param {body} body เนื้อหาข้อมูลที่ต้อง save ลง database
     */
    put(body: any): Promise<any>;
    /**
     * เก็บข้อมูลแบบ update item จาก DynamoDB
     * @param {object<[key: string]: string>} keys key หลักในการเรียกข้อมูลจากตาราง partitionKey หรือ compositeKey
     * @param {body} body เนื้อหาข้อมูลที่ต้อง save ลง database
     */
    update(keys: {
        [key: string]: string;
    }, body: any): Promise<any>;
    patch(keys: {
        [key: string]: string;
    }, body: any): Promise<any>;
    /**
     * ลบข้อมูล item จาก DynamoDB
     * @param {object<[key: string]: string>} keys key หลักในการเรียกข้อมูลจากตาราง partitionKey หรือ compositeKey
     */
    delete(keys: any): Promise<any>;
    /**
     * Query Operation บน DynamoDB แบบรองรับ recursive รองรับได้เฉพาะ Attribute lv 1
     * @param {ScanOptions} options รายละเอียดของ Query Param
     */
    query(options?: QueryOptions): Promise<any[]>;
    /**
     * Scan Operation บน DynamoDB แบบรองรับ recursive
     * @param {string} options รายละเอียดของ Scan Param
     */
    scan(options?: ScanOptions): Promise<any[]>;
    static convertScanQueryOptions(options: ScanQueryOptions): Partial<ScanOptions>;
    /**
     * Recursive function for Scan and Query operation
     * @param fnName
     * @param param
     * @param {ScanOptions} options
     */
    private recursiveOperation;
    private createValueKey;
    private createKeyName;
    private createConditionExpression;
    private convertExpressionValue;
    private convertExpressionNames;
    private convertAttributeNames;
    private convertProjectionName;
    private static mapConditionToString;
}
